#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "mu.h"


#define MY_SIG (SIGRTMIN + 2)   /* 36 */


struct task {
    char *name;
    struct itimerspec itimerspec;
    timer_t timer_id;
};


static struct task *
task_new(const char *name, unsigned int secs)
{
    struct task *task = mu_zalloc(sizeof(*task));
    struct sigevent ev;
    int err;

    task->name = mu_strdup(name);
    task->itimerspec.it_interval.tv_sec = secs;

    memset(&ev, 0x00, sizeof(ev));
    ev.sigev_notify = SIGEV_SIGNAL;
    ev.sigev_signo = MY_SIG;
    ev.sigev_value.sival_ptr = task;
    
    err = timer_create(CLOCK_REALTIME, &ev, &task->timer_id);
    if (err == -1)
        mu_die_errno(errno, "timer_create");

    return task;
}


static void
task_arm(struct task *task)
{
    task->itimerspec.it_value = task->itimerspec.it_interval;
    timer_settime(task->timer_id, 0, &task->itimerspec, NULL);
}


static void
task_free(struct task *task)
{
    int err;

    err = timer_delete(task->timer_id);
    if (err == -1)
        mu_die_errno(errno, "timer_delete");

    free(task->name);
    free(task);
}


int
main(void)
{
    sigset_t set;
    siginfo_t info;
    int signo;
    struct task *t1, *t2, *task;
    time_t now;

    t1 = task_new("foo", 4);
    t2 = task_new("bar", 8);
    
    sigemptyset(&set);
    sigaddset(&set, MY_SIG);
    sigaddset(&set, SIGINT);
    sigprocmask(SIG_BLOCK, &set, NULL);

    task_arm(t1);
    task_arm(t2);

    while (1) {
        signo = sigwaitinfo(&set, &info);
        if (signo == SIGINT)
            break;

        task = info.si_value.sival_ptr;

        now = time(NULL);
        printf("%ld %s\n", (long)now, task->name);
    }

    task_free(t1);
    task_free(t2);

    return 0;
}
